// SETI_Monitor - Monitoring SETI Client software for Linux
//
// 1999-2000 by Christophe Lucas <c.lucas@ifrance.com>
//
// This file is subject to the terms and conditions of the GNU General
// Public License.  See the file COPYING in the main directory of the
// SETI_Monitor distribution for more details.
//
// Version 0.0.1
//

#include "csock.h"

extern int errno;

void error_socket( const char* format , ... )
{
    va_list args;
    va_start(args , format );
    vfprintf(stderr , format , args );
    va_end(args);
    exit(1);
}
    
    
    
int creat_socket( const char* service , const char* transport , struct sockaddr_in* sin )
{
    //Structures permettant de rcuprer des infos numriques
    // partir des noms de service (nport) et protocole
    struct servent* iservice;
    struct protoent* itransport;
    int s , type , proto ;
    
    //Initialisation de la structure socket
    memset( sin , 0 , sizeof(*sin) ) ;
    sin->sin_family = AF_INET ;
    
    //Recherche du numro de port
    sin->sin_port = htons( atoi(service) ) ;
    if( !sin->sin_port )
    {
	//Si c'est pas un numero de port on recherche dans /etc/services
	iservice=getservbyname(service , transport );
	if( !iservice )
	{
	    error_socket("Service non enregistr: %s",service ); 
	}
	sin->sin_port = iservice ->s_port;
    }
    
    //Recherche du numro de protocole
    itransport = getprotobyname(transport);
    if( !itransport )
    {
	error_socket("Protocole non enregistr: %s",transport);
    }
    proto = itransport->p_proto ;
    
    //Diffrenciation entre "udp" et "tcp"
    if( !strcmp( transport , "udp" ) )
	type = SOCK_DGRAM ; // UDP
    else
	type = SOCK_STREAM; // TCP
    
    s = socket( AF_INET, type , proto );
    if( s < 0 )
    {
	error_socket("Creation de socket impossible: %s",strerror(errno) );
    }
    
    return s;
    
}//creat_socket

// Cration d'une socket pour un client
int client_socket( const char* host , const char* service , const char* transport )
{
    //strusture dcrivant la socket
    struct sockaddr_in sin;
    struct hostent* ihost;
    
    int s;
    
    //cration d'une socket
    s = creat_socket(service , transport , &sin );
    
    //Recherche de l'adresse IP de host
    sin.sin_addr.s_addr = inet_addr(host) ;
    
    if( sin.sin_addr.s_addr == INADDR_NONE )
    {
	//Si host est un nom et non pas une addr IP
	//On recherche dans /etc/hosts et dns
	ihost = gethostbyname( host );
	if( !ihost )
	    error_socket("Hte inconnu:%s",host );
	    
	memcpy(&sin.sin_addr , ihost->h_addr , ihost->h_length );
	
    }
    
    //connexion de la socket
    if( connect( s , (struct sockaddr* ) &sin , sizeof(sin) ) < 0 )
	error_socket("Impossible de se connecter  %s[%s] : %s" , host, service , strerror(errno) );
	
    return s;	
}

// Cration d'une socket pour un serveur
int server_socket( const char* service , const char* transport , int lqueue )
{
    struct sockaddr_in sin;
    
    int s;
    
    s = creat_socket( service , transport , &sin );
    
    sin.sin_addr.s_addr = INADDR_ANY ;
     
    //Allocation du numro de port
    if( bind( s , (struct sockaddr*) &sin , sizeof( sin ) ) < 0 )
    {
	error_socket("Impossible de lier la socket  %s: %s",service , strerror(errno) );
    }
    
    //on place la socket en attente si transport n'est pas udp
    if( strcmp(transport, "udp" ) && listen( s , lqueue )<0 )
    {
	error_socket("Mode passif impossible sur %s: %s",service,strerror(errno) );
    }
    
    return s;
    
}